<?php
/*
* NOTICE OF LICENSE
*
* This file is part of blocktopviewed prestashop module.
*
* blocktopviewed is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* blocktopviewed is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with blocktopviewed.  If not, see <http://www.gnu.org/licenses/>.
*
*  @author Duarte Rocha <duarte.rocha@gmail.com>
*  @copyright  2013 Duarte Rocha
*  @license  http://www.gnu.org/licenses/gpl.html 
*
* 
* This module will present the most viewd products on the store
*
*/

// Checks the existence of the prestashop Constant, if it does not exist, don't load the module
if (!defined('_PS_VERSION_'))
  exit;

// Module Class
class Blocktopviewed extends Module
{
  private $_query =  null;
  private $limit =  10 ;

  // Class construct
  public function __construct()
  {
    $this->name = 'blocktopviewed';
    $this->tab = 'front_office_features';
    $this->version = '0.2';
    $this->author = 'Duarte Rocha <duarte.rocha@gmail.com';
    $this->need_instance = 0;
    $this->ps_versions_compliancy = array('min' => '1.5', 'max' => '1.6'); 

    parent::__construct();

    $this->displayName = $this->l('Most Viewed Products');
    $this->description = $this->l('This module presents the list of the most viewed products by clients.');

    $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

    if (Configuration::get('BLOCKTOPVIEWED_LIMIT'))       
      $this->limit = Configuration::get('BLOCKTOPVIEWED_LIMIT');
  }

  public function install()
  {
    if (Shop::isFeatureActive())
      Shop::setContext(Shop::CONTEXT_ALL);

    // install the module and check if
    // the module is tied to the header (because of css) and
    // set a value to BLOCKVETOP NAME and LIMIT
    return parent::install() &&
      $this->registerHook('header') &&
      $this->registerHook('rightColumn') &&
      Configuration::updateValue('BLOCKTOPVIEWED_LIMIT', 10);
  }
  
  // Uninstall Module 
  public function uninstall()
  {
    return parent::uninstall() && Configuration::deleteByName('BLOCKTOPVIEWED_NAME');
    return parent::uninstall() && Configuration::deleteByName('BLOCKTOPVIEWED_LIMIT');
  }
   
  // Create Configuration Page for the module
  // Only 1 parameter will be available (nº off products to list)
  public function getContent()
  {
    $output = null;

    if (Tools::isSubmit('submit'.$this->name)) {
      $top_viewed_limit = strval(Tools::getValue('BLOCKTOPVIEWED_LIMIT'));
      if (!$top_viewed_limit  || empty($top_viewed_limit) || !Validate::isGenericName($top_viewed_limit))
        $output .= $this->displayError( $this->l('Invalid Configuration value') );
      else
      {
        Configuration::updateValue('BLOCKTOPVIEWED_LIMIT', $top_viewed_limit);
        $output .= $this->displayConfirmation($this->l('Settings updated'));
      }
    }
    return $output.$this->displayForm();
  }

  public function displayForm()
  {
      // Get default Language
      $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
       
      // Init Fields form array
      $fields_form[0]['form'] = array(
          'legend' => array(
              'title' => $this->l('Settings'),
          ),
          'input' => array(
              array(
                  'type' => 'text',
                  'label' => $this->l('How many products should be listed?'),
                  'name' => 'BLOCKTOPVIEWED_LIMIT',
                  'size' => 2,
                  'required' => true
              )
          ),
          'submit' => array(
              'title' => $this->l('Save'),
              'class' => 'button'
          )
      );
       
      $helper = new HelperForm();
       
      // Module, token and currentIndex
      $helper->module = $this;
      $helper->name_controller = $this->name;
      $helper->token = Tools::getAdminTokenLite('AdminModules');
      $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
       
      // Language
      $helper->default_form_language = $default_lang;
      $helper->allow_employee_form_lang = $default_lang;
       
      // Title and toolbar
      $helper->title = $this->displayName;
      $helper->show_toolbar = true;        // false -> remove toolbar
      $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
      $helper->submit_action = 'submit'.$this->name;
      $helper->toolbar_btn = array(
          'save' =>
          array(
              'desc' => $this->l('Save'),
              'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
              '&token='.Tools::getAdminTokenLite('AdminModules'),
          ),
          'back' => array(
              'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
              'desc' => $this->l('Back to list')
          )
      );
       
      // Load current value
      $helper->fields_value['BLOCKTOPVIEWED_LIMIT'] = Configuration::get('BLOCKTOPVIEWED_LIMIT');
       
      return $helper->generateForm($fields_form);
  }

  // Header of pages (<head></head>), used to insert javascript, css, etc...
  public function hookDisplayHeader() {
    // Add css file
    $this->context->controller->addCSS($this->_path.'css/blocktopviewed.css', 'all');
  }

  public function hookdisplayRightColumn ($params)
  {
    $results = array();

    //$limit = (int) Configuration::get('BLOCKTOPVIEWED_LIMIT') ;
    //$this->limit;

    $this->_query = 'SELECT p.id_product, SUM(pv.counter) AS total_views
      FROM
        '._DB_PREFIX_.'product AS p
        LEFT JOIN '._DB_PREFIX_.'page AS pa ON p.id_product = pa.id_object
        LEFT JOIN '._DB_PREFIX_.'page_viewed AS pv ON pa.id_page = pv.id_page
      WHERE
        pa.id_page_type = ('.(int)Page::getPageTypeByName('product').')
      GROUP BY p.id_product
      ORDER BY total_views DESC
      LIMIT ' . $this->limit ;

    if( $db = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($this->_query) )
    {
      // For every resulting sql row
      foreach( $db as $result)
      {
        // For each entry, build product object
        $product =  new Product($result['id_product'], false, intval($params['cookie']->id_lang));
        // Cover image ID
        $image_id = $product->getCover($product->id);
        // Add image id to Product Object
        $product->id_image = $image_id['id_image'];
        // Added product object to the end of the results array 
        array_push($results, $product);
      }
      // assign array with results to smarty template
      $most_viewed_error = FALSE;

    }
    else
    {
      // if there was an sql error, send notice to smarty
      $most_viewed_error = TRUE ;
    }
    $this->context->smarty->assign( 
      array(
        'most_viewed_error' => $most_viewed_error,
        'most_viewed_results' => $results,
      )
    );

    // Return the html to print
    return $this->display(__FILE__, 'blocktopviewed.tpl');
  }

  public function hookdisplayLeftColumn ($params)
  {
    return $this->hookdisplayRightColumn ($params) ;
  }

}
?>
